/**
 * DevExtreme (ui/gantt/ui.gantt.js)
 * Version: 19.2.1 (build 19228-2142)
 * Build date: Fri Aug 16 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _get = function get(object, property, receiver) {
    if (null === object) {
        object = Function.prototype
    }
    var desc = Object.getOwnPropertyDescriptor(object, property);
    if (void 0 === desc) {
        var parent = Object.getPrototypeOf(object);
        if (null === parent) {
            return
        } else {
            return get(parent, property, receiver)
        }
    } else {
        if ("value" in desc) {
            return desc.value
        } else {
            var getter = desc.get;
            if (void 0 === getter) {
                return
            }
            return getter.call(receiver)
        }
    }
};
var _renderer = require("../../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _ui = require("../widget/ui.widget");
var _ui2 = _interopRequireDefault(_ui);
var _component_registrator = require("../../core/component_registrator");
var _component_registrator2 = _interopRequireDefault(_component_registrator);
var _dom_adapter = require("../../core/dom_adapter");
var _dom_adapter2 = _interopRequireDefault(_dom_adapter);
var _data = require("../../core/utils/data");
var _data2 = _interopRequireDefault(_data);
var _events_engine = require("../../events/core/events_engine");
var _events_engine2 = _interopRequireDefault(_events_engine);
var _pointer = require("../../events/pointer");
var _pointer2 = _interopRequireDefault(_pointer);
var _utils = require("../../events/utils");
var _uiGantt = require("./ui.gantt.view");
var _tree_list = require("../tree_list");
var _tree_list2 = _interopRequireDefault(_tree_list);
var _extend = require("../../core/utils/extend");
var _window = require("../../core/utils/window");
var _uiGanttData = require("./ui.gantt.data.option");
var _uiGanttData2 = _interopRequireDefault(_uiGanttData);

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _possibleConstructorReturn(self, call) {
    if (!self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return call && ("object" === typeof call || "function" === typeof call) ? call : self
}

function _inherits(subClass, superClass) {
    if ("function" !== typeof superClass && null !== superClass) {
        throw new TypeError("Super expression must either be null or a function, not " + typeof superClass)
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            enumerable: false,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass
    }
}
var GANTT_CLASS = "dx-gantt";
var GANTT_SPLITTER_CLASS = "dx-gantt-splitter";
var GANTT_SPLITTER_TRANSPARENT_CLASS = "dx-gantt-splitter-transparent";
var GANTT_SPLITTER_BORDER_CLASS = "dx-gantt-splitter-border";
var GANTT_VIEW_CLASS = "dx-gantt-view";
var GANTT_COLLAPSABLE_ROW = "dx-gantt-collapsable-row";
var GANTT_DEFAULT_ROW_HEIGHT = 34;
var GANTT_MODULE_NAMESPACE = "dxGanttResizing";
var GANTT_POINTER_DOWN_EVENT_NAME = (0, _utils.addNamespace)(_pointer2.default.down, GANTT_MODULE_NAMESPACE);
var GANTT_POINTER_MOVE_EVENT_NAME = (0, _utils.addNamespace)(_pointer2.default.move, GANTT_MODULE_NAMESPACE);
var GANTT_POINTER_UP_EVENT_NAME = (0, _utils.addNamespace)(_pointer2.default.up, GANTT_MODULE_NAMESPACE);
var GANTT_WINDOW_RESIZE_EVENT_NAME = (0, _utils.addNamespace)("resize", GANTT_MODULE_NAMESPACE);
var Gantt = function(_Widget) {
    _inherits(Gantt, _Widget);

    function Gantt() {
        _classCallCheck(this, Gantt);
        return _possibleConstructorReturn(this, (Gantt.__proto__ || Object.getPrototypeOf(Gantt)).apply(this, arguments))
    }
    _createClass(Gantt, [{
        key: "_init",
        value: function() {
            _get(Gantt.prototype.__proto__ || Object.getPrototypeOf(Gantt.prototype), "_init", this).call(this);
            this._refreshDataSource("tasks");
            this._refreshDataSource("dependencies");
            this._refreshDataSource("resources");
            this._refreshDataSource("resourceAssignments")
        }
    }, {
        key: "_initMarkup",
        value: function() {
            _get(Gantt.prototype.__proto__ || Object.getPrototypeOf(Gantt.prototype), "_initMarkup", this).call(this);
            this.$element().addClass(GANTT_CLASS);
            this._$treeListWrapper = (0, _renderer2.default)("<div>").appendTo(this.$element());
            this._$treeList = (0, _renderer2.default)("<div>").appendTo(this._$treeListWrapper);
            this._$splitter = (0, _renderer2.default)("<div>").addClass(GANTT_SPLITTER_CLASS).addClass(GANTT_SPLITTER_TRANSPARENT_CLASS).appendTo(this.$element());
            this._$splitterBorder = (0, _renderer2.default)("<div>").addClass(GANTT_SPLITTER_BORDER_CLASS).appendTo(this.$element());
            this._$ganttView = (0, _renderer2.default)("<div>").addClass(GANTT_VIEW_CLASS).appendTo(this.$element())
        }
    }, {
        key: "_render",
        value: function() {
            this._renderTreeList();
            this._renderSplitter();
            this._detachEventHandlers();
            this._attachEventHandlers()
        }
    }, {
        key: "_renderTreeList",
        value: function() {
            var _this2 = this;
            this._$treeListWrapper.width(this.option("treeListWidth"));
            this._$treeList.width(this.option("treeListWidth"));
            this._treeList = this._createComponent(this._$treeList, _tree_list2.default, {
                dataSource: this._tasks,
                columns: this.option("columns"),
                columnResizingMode: "widget",
                height: "100%",
                selection: {
                    mode: "single"
                },
                sorting: {
                    mode: "none"
                },
                scrolling: {
                    showScrollbar: "onHover",
                    mode: "standard"
                },
                allowColumnResizing: true,
                autoExpandAll: true,
                showRowLines: true,
                onContentReady: function(e) {
                    _this2._onTreeListContentReady(e)
                },
                onSelectionChanged: function(e) {
                    return _this2._ganttView.selectTask(e.currentSelectedRowKeys[0])
                },
                onRowCollapsed: function(e) {
                    return _this2._ganttView.changeTaskExpanded(e.key, false)
                },
                onRowExpanded: function(e) {
                    return _this2._ganttView.changeTaskExpanded(e.key, true)
                },
                onRowPrepared: function(e) {
                    _this2._onTreeListRowPrepared(e)
                }
            })
        }
    }, {
        key: "_renderSplitter",
        value: function() {
            this._$splitter.css("left", this.option("treeListWidth"))
        }
    }, {
        key: "_detachEventHandlers",
        value: function() {
            var document = _dom_adapter2.default.getDocument();
            _events_engine2.default.off(this._$splitter, GANTT_POINTER_DOWN_EVENT_NAME);
            _events_engine2.default.off(document, GANTT_POINTER_MOVE_EVENT_NAME);
            _events_engine2.default.off(document, GANTT_POINTER_UP_EVENT_NAME);
            _events_engine2.default.off((0, _window.getWindow)(), GANTT_WINDOW_RESIZE_EVENT_NAME)
        }
    }, {
        key: "_attachEventHandlers",
        value: function() {
            var document = _dom_adapter2.default.getDocument();
            _events_engine2.default.on(this._$splitter, GANTT_POINTER_DOWN_EVENT_NAME, this._startResizingHandler.bind(this));
            _events_engine2.default.on(document, GANTT_POINTER_MOVE_EVENT_NAME, this._moveSplitterHandler.bind(this));
            _events_engine2.default.on(document, GANTT_POINTER_UP_EVENT_NAME, this._endResizingHandler.bind(this));
            _events_engine2.default.on((0, _window.getWindow)(), GANTT_WINDOW_RESIZE_EVENT_NAME, this._windowResizeHandler.bind(this))
        }
    }, {
        key: "_initGanttView",
        value: function() {
            var _this3 = this;
            if (this._ganttView) {
                return
            }
            this._ganttView = this._createComponent(this._$ganttView, _uiGantt.GanttView, {
                height: this._treeList._$element.get(0).offsetHeight,
                rowHeight: this._getTreeListRowHeight(),
                tasks: this._tasks,
                dependencies: this._dependencies,
                resources: this._resources,
                resourceAssignments: this._resourceAssignments,
                showResources: this.option("showResources"),
                taskTitlePosition: this.option("taskTitlePosition"),
                onSelectionChanged: function(e) {
                    _this3._onGanttViewSelectionChanged(e)
                },
                onScroll: function(e) {
                    _this3._onGanttViewScroll(e)
                }
            })
        }
    }, {
        key: "_startResizingHandler",
        value: function(e) {
            e.preventDefault();
            this._$splitter.removeClass(GANTT_SPLITTER_TRANSPARENT_CLASS);
            this._isSplitterMove = true;
            this._splitterLastLeftPos = e.clientX
        }
    }, {
        key: "_moveSplitterHandler",
        value: function(e) {
            if (this._isSplitterMove) {
                var newLeftPos = this._$splitter.position().left - this._splitterLastLeftPos + e.clientX;
                newLeftPos = Math.max(0, newLeftPos);
                newLeftPos = Math.min(newLeftPos, this.$element().width());
                this._splitterLastLeftPos = e.clientX;
                this._updateWidth(newLeftPos)
            }
        }
    }, {
        key: "_endResizingHandler",
        value: function() {
            if (this._isSplitterMove) {
                this._$splitter.addClass(GANTT_SPLITTER_TRANSPARENT_CLASS);
                this._isSplitterMove = false;
                this.option("treeListWidth", this._$splitter.position().left)
            }
        }
    }, {
        key: "_windowResizeHandler",
        value: function() {
            this._updateWidth(this.option("treeListWidth"))
        }
    }, {
        key: "_onTreeListContentReady",
        value: function(e) {
            if (e.component.getDataSource()) {
                this._initGanttView();
                this._initScrollSync(e.component)
            }
        }
    }, {
        key: "_onTreeListRowPrepared",
        value: function(e) {
            if ("data" === e.rowType && e.node.children.length > 0) {
                (0, _renderer2.default)(e.rowElement).addClass(GANTT_COLLAPSABLE_ROW)
            }
        }
    }, {
        key: "_onGanttViewSelectionChanged",
        value: function(e) {
            this._setTreeListOption("selectedRowKeys", [e.id])
        }
    }, {
        key: "_onGanttViewScroll",
        value: function(e) {
            var treeListScrollable = this._treeList.getScrollable();
            if (treeListScrollable) {
                var diff = e.scrollTop - treeListScrollable.scrollTop();
                if (0 !== diff) {
                    treeListScrollable.scrollBy({
                        left: 0,
                        top: diff
                    })
                }
            }
        }
    }, {
        key: "_onTreeListScroll",
        value: function(treeListScrollView) {
            var ganttViewTaskAreaContainer = this._ganttView.getTaskAreaContainer();
            if (ganttViewTaskAreaContainer.scrollTop !== treeListScrollView.component.scrollTop()) {
                ganttViewTaskAreaContainer.scrollTop = treeListScrollView.component.scrollTop()
            }
        }
    }, {
        key: "_updateGanttView",
        value: function() {
            this._ganttView.option("tasks", this._getTasks())
        }
    }, {
        key: "_getTasks",
        value: function() {
            var _this4 = this;
            this._tasks.forEach(function(t, i) {
                t.expanded = _this4._treeList.isRowExpanded(i + 1)
            });
            return this._tasks
        }
    }, {
        key: "_initScrollSync",
        value: function(treeList) {
            var _this5 = this;
            var treeListScrollable = treeList.getScrollable();
            if (treeListScrollable) {
                treeListScrollable.off("scroll");
                treeListScrollable.on("scroll", function(e) {
                    _this5._onTreeListScroll(e)
                })
            }
        }
    }, {
        key: "_getTreeListRowHeight",
        value: function() {
            var $row = this._treeList._$element.find(".dx-row-lines");
            return $row ? $row.last().outerHeight() : GANTT_DEFAULT_ROW_HEIGHT
        }
    }, {
        key: "_updateWidth",
        value: function(treeListWidth) {
            if (!(0, _window.hasWindow)()) {
                return
            }
            var splitterBorderWidth = this._$splitterBorder.outerWidth();
            treeListWidth = Math.min(treeListWidth, this.$element().width() - splitterBorderWidth);
            this._$treeListWrapper.width(treeListWidth);
            this._$treeList.width(treeListWidth);
            this._$splitter.css("left", treeListWidth);
            this._ganttView && this._ganttView.setWidth(this.$element().width() - treeListWidth - splitterBorderWidth)
        }
    }, {
        key: "_setGanttViewOption",
        value: function(optionName, value) {
            this._ganttView && this._ganttView.option(optionName, value)
        }
    }, {
        key: "_setTreeListOption",
        value: function(optionName, value) {
            this._treeList && this._treeList.option(optionName, value)
        }
    }, {
        key: "_refreshDataSource",
        value: function(name) {
            var _this6 = this;
            var dataOption = this["_" + name + "Option"];
            if (dataOption) {
                dataOption._disposeDataSource();
                delete this["_" + name + "Option"];
                delete this["_" + name]
            }
            if (this.option(name + ".dataSource")) {
                dataOption = new _uiGanttData2.default(name, function(name, data) {
                    _this6._dataSourceChanged(name, data)
                });
                dataOption.option("dataSource", this.option(name + ".dataSource"));
                dataOption._refreshDataSource();
                this["_" + name + "Option"] = dataOption
            }
        }
    }, {
        key: "_compileGettersByOption",
        value: function(optionName) {
            var getters = {};
            var optionValue = this.option(optionName);
            for (var field in optionValue) {
                var exprMatches = field.match(/(\w*)Expr/);
                if (exprMatches) {
                    getters[exprMatches[1]] = _data2.default.compileGetter(optionValue[exprMatches[0]])
                }
            }
            return getters
        }
    }, {
        key: "_prepareMapHandler",
        value: function(getters) {
            return function(data) {
                return Object.keys(getters).reduce(function(previous, key) {
                    var resultKey = "key" === key ? "id" : key;
                    previous[resultKey] = getters[key](data);
                    return previous
                }, {})
            }
        }
    }, {
        key: "_dataSourceChanged",
        value: function(dataSourceName, data) {
            var getters = this._compileGettersByOption(dataSourceName);
            var mappedData = data.map(this._prepareMapHandler(getters));
            this["_" + dataSourceName] = mappedData;
            this._setGanttViewOption(dataSourceName, mappedData);
            if ("tasks" === dataSourceName) {
                this._setTreeListOption("dataSource", mappedData)
            }
        }
    }, {
        key: "_clean",
        value: function() {
            delete this._ganttView;
            this._detachEventHandlers();
            _get(Gantt.prototype.__proto__ || Object.getPrototypeOf(Gantt.prototype), "_clean", this).call(this)
        }
    }, {
        key: "_getDefaultOptions",
        value: function() {
            return (0, _extend.extend)(_get(Gantt.prototype.__proto__ || Object.getPrototypeOf(Gantt.prototype), "_getDefaultOptions", this).call(this), {
                tasks: {
                    dataSource: null,
                    keyExpr: "id",
                    parentIdExpr: "parentId",
                    startExpr: "start",
                    endExpr: "end",
                    progressExpr: "progress",
                    titleExpr: "title"
                },
                dependencies: {
                    dataSource: null,
                    keyExpr: "id",
                    predecessorIdExpr: "predecessorId",
                    successorIdExpr: "successorId",
                    typeExpr: "type"
                },
                resources: {
                    dataSource: null,
                    keyExpr: "id",
                    textExpr: "text"
                },
                resourceAssignments: {
                    dataSource: null,
                    keyExpr: "id",
                    taskIdExpr: "taskId",
                    resourceIdExpr: "resourceId"
                },
                treeListWidth: 300,
                showResources: true,
                taskTitlePosition: "inside"
            })
        }
    }, {
        key: "_optionChanged",
        value: function(args) {
            switch (args.name) {
                case "tasks":
                    this._refreshDataSource("tasks");
                    break;
                case "dependencies":
                    this._refreshDataSource("dependencies");
                    break;
                case "resources":
                    this._refreshDataSource("resources");
                    break;
                case "resourceAssignments":
                    this._refreshDataSource("resourceAssignments");
                    break;
                case "treeListWidth":
                    this._updateWidth(args.value);
                    break;
                case "showResources":
                    this._setGanttViewOption("showResources", args.value);
                    break;
                case "taskTitlePosition":
                    this._setGanttViewOption("taskTitlePosition", args.value);
                    break;
                default:
                    _get(Gantt.prototype.__proto__ || Object.getPrototypeOf(Gantt.prototype), "_optionChanged", this).call(this, args)
            }
        }
    }]);
    return Gantt
}(_ui2.default);
(0, _component_registrator2.default)("dxGantt", Gantt);
module.exports = Gantt;
